# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import io
import json
from typing import Dict, List, Optional

from .. import (
    app,
    config,
    convert,
    exceptions,
    experiment_api,
    experiment_info,
    llm_result,
    logging_messages,
)
from ..types import JSONEncodable
from . import chain, state


@exceptions.filter(allow_raising=config.raising_enabled(), summary=app.SUMMARY)
def start_chain(
    inputs: Dict[str, JSONEncodable],
    api_key: Optional[str] = None,
    workspace: Optional[str] = None,
    project: Optional[str] = None,
    metadata: Optional[Dict[str, Dict[str, JSONEncodable]]] = None,
    tags: Optional[List[str]] = None,
) -> None:
    """
    Creates global Chain object that tracks created Spans.
    Args:
        inputs: Dict[str, JSONEncodable] (required) chain inputs.
        workspace: str (optional) comet workspace to use for logging.
        project: str (optional) project name to create in comet workspace.
        tags: List[str] (optional), user-defined tags attached to a prompt call.
        api_key: str (optional) comet API key.
        metadata: Dict[str, Dict[str, JSONEncodable]] (optional) user-defined
            dictionary with additional metadata to the call.
        tags: List[str] (optional) user-defined tags attached to the chain
    """

    MESSAGE = """
    CometLLM requires an API key. Please provide it as the
    api_key argument to comet_llm.start_chain or as an environment
    variable named COMET_API_KEY
    """

    experiment_info_ = experiment_info.get(
        api_key,
        workspace,
        project,
        api_key_not_found_message=MESSAGE,
    )
    global_chain = chain.Chain(
        inputs=inputs,
        metadata=metadata,
        experiment_info=experiment_info_,
        tags=tags,
    )
    state.set_global_chain(global_chain)


@exceptions.filter(allow_raising=config.raising_enabled(), summary=app.SUMMARY)
def end_chain(
    outputs: Dict[str, JSONEncodable],
    metadata: Optional[Dict[str, JSONEncodable]] = None,
) -> llm_result.LLMResult:
    """
    Commits global chain and logs the result to Comet.
    Args:
        outputs: Dict[str, JSONEncodable] (required) chain outputs.
        metadata: Dict[str, Dict[str, JSONEncodable]] (optional) user-defined
            dictionary with additional metadata to the call. This metadata
            will be deep merged with the metadata passed to start_chain if
            it was provided.
        tags: List[str] (optional) user-defined tags attached to the chain

    Returns: LLMResult
    """
    global_chain = state.get_global_chain()
    if global_chain is None:
        raise exceptions.CometLLMException(
            logging_messages.GLOBAL_CHAIN_NOT_INITIALIZED % "`end_chain`"
        )

    global_chain.set_outputs(outputs=outputs, metadata=metadata)
    return log_chain(global_chain)


def log_chain(chain: chain.Chain) -> llm_result.LLMResult:
    chain_data = chain.as_dict()

    experiment_info_ = chain.experiment_info
    experiment_api_ = experiment_api.ExperimentAPI.create_new(
        api_key=experiment_info_.api_key,
        workspace=experiment_info_.workspace,
        project_name=experiment_info_.project_name,
    )

    if chain.tags is not None:
        experiment_api_.log_tags(chain.tags)

    experiment_api_.log_asset_with_io(
        name="comet_llm_data.json",
        file=io.StringIO(json.dumps(chain_data)),
        asset_type="llm_data",
    )

    experiment_api_.log_metric(
        name="chain_duration", value=chain_data["chain_duration"]
    )

    parameters = convert.chain_metadata_to_flat_parameters(chain_data["metadata"])
    for name, value in parameters.items():
        experiment_api_.log_parameter(name, value)

    for name, value in chain.others.items():
        experiment_api_.log_other(name, value)

    app.SUMMARY.add_log(experiment_api_.project_url, "chain")

    return llm_result.LLMResult(
        id=experiment_api_.id, project_url=experiment_api_.project_url
    )



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import functools
import inspect
from typing import Callable

from . import callable_extenders, callback_runners


def wrap(
    original: Callable, callbacks: callable_extenders.CallableExtenders
) -> Callable:
    original = _unbound_if_classmethod(original)

    @functools.wraps(original)
    def wrapped(*args, **kwargs):  # type: ignore
        args, kwargs = callback_runners.run_before(
            callbacks.before, original, *args, **kwargs
        )
        try:
            result = original(*args, **kwargs)
        except Exception as exception:
            callback_runners.run_after_exception(
                callbacks.after_exception, original, exception, *args, **kwargs
            )
            raise exception

        result = callback_runners.run_after(
            callbacks.after, original, result, *args, **kwargs
        )

        return result

    return wrapped


def _unbound_if_classmethod(original: Callable) -> Callable:
    if hasattr(original, "__self__") and inspect.isclass(original.__self__):
        # when original is classmethod, mypy doesn't consider it as a callable.
        original = original.__func__  # type: ignore

    return original



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import functools
from typing import TYPE_CHECKING, Any, Callable, Optional

from comet_llm.chains import thread_context_registry

if TYPE_CHECKING:
    from comet_llm.chains.chain import Chain
    from comet_llm.chains.span import Span


class OpenAIContext:
    def __init__(self) -> None:
        self._thread_context_registry = thread_context_registry.ThreadContextRegistry()

    @property
    def chain(self) -> Optional["Chain"]:
        return self._thread_context_registry.get("chain")  # type: ignore

    @chain.setter
    def chain(self, value: "Chain") -> None:
        self._thread_context_registry.add("chain", value)

    @property
    def span(self) -> Optional["Span"]:
        return self._thread_context_registry.get("span")  # type: ignore

    @span.setter
    def span(self, value: "Span") -> None:
        self._thread_context_registry.add("span", value)

    def clear(self) -> None:
        self._thread_context_registry.add("span", None)
        self._thread_context_registry.add("chain", None)


def clear_on_end(function: Callable) -> Callable:
    @functools.wraps(function)
    def wrapped(*args, **kwargs) -> Any:  # type: ignore
        try:
            return function(*args, **kwargs)
        finally:
            CONTEXT.clear()

    return wrapped


CONTEXT = OpenAIContext()



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import functools
import logging
from typing import TYPE_CHECKING, Any, Callable, Optional

from comet_llm import logging as comet_logging

if TYPE_CHECKING:
    from comet_llm import summary

LOGGER = logging.getLogger(__name__)


def filter(
    allow_raising: bool, summary: Optional["summary.Summary"] = None
) -> Callable:
    def decorator(function: Callable) -> Callable:
        @functools.wraps(function)
        def wrapper(*args, **kwargs) -> Any:  # type: ignore
            try:
                return function(*args, **kwargs)
            except Exception as exception:
                if summary is not None:
                    summary.increment_failed()

                if allow_raising:
                    raise

                if getattr(exception, "log_message_once", False):
                    comet_logging.log_once_at_level(
                        LOGGER,
                        logging.ERROR,
                        str(exception),
                        exc_info=True,
                        extra={"show_traceback": True},
                    )
                else:
                    LOGGER.error(
                        str(exception),
                        exc_info=True,
                        extra={"show_traceback": True},
                    )

        return wrapper

    return decorator



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import calendar
import datetime
from typing import Optional

_JAN1_2010 = datetime.datetime(2010, 1, 1).timestamp()
_JAN1_2030 = datetime.datetime(2030, 1, 1).timestamp()


class Timer:
    def __init__(self) -> None:
        self._start_timestamp: Optional[int] = None
        self._end_timestamp: Optional[int] = None
        self._duration: Optional[int] = None

    def start(self) -> None:
        self._start_timestamp = local_timestamp()
        self._end_timestamp = None
        self._duration = None

    def stop(self) -> None:
        assert self._start_timestamp is not None
        self._end_timestamp = local_timestamp()
        self._duration = self._end_timestamp - self._start_timestamp

    @property
    def start_timestamp(self) -> Optional[int]:
        return self._start_timestamp

    @property
    def end_timestamp(self) -> Optional[int]:
        return self._end_timestamp

    @property
    def duration(self) -> Optional[int]:
        return self._duration


def is_valid_timestamp_seconds(timestamp: float) -> bool:
    if timestamp < _JAN1_2010 or timestamp > _JAN1_2030:
        return False

    return True


def local_timestamp() -> int:
    now = datetime.datetime.utcnow()
    timestamp_in_seconds = calendar.timegm(now.timetuple()) + (now.microsecond / 1e6)
    timestamp_in_milliseconds = int(timestamp_in_seconds * 1000)
    return timestamp_in_milliseconds



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

from . import app, autologgers, config, logging
from .config import init, is_ready

if config.comet_disabled():
    from .dummy_api import Span, end_chain, log_prompt, start_chain  # type: ignore
else:
    from .api import flush, log_user_feedback
    from .chains.api import end_chain, start_chain
    from .chains.span import Span
    from .prompts.api import log_prompt


__all__ = [
    "log_prompt",
    "start_chain",
    "end_chain",
    "Span",
    "init",
    "is_ready",
    "log_user_feedback",
    "flush",
]

logging.setup()
app.register_summary_print()
autologgers.patch()



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import inspect
from types import ModuleType
from typing import TYPE_CHECKING, Any

from . import wrapper

if TYPE_CHECKING:  # pragma: no cover
    from . import module_extension

# _get_object and _set_object copied from comet_ml.monkeypatching almost without any changes.


def _get_object(module: ModuleType, callable_path: str) -> Any:
    current_object = module

    for part in callable_path:
        try:
            current_object = getattr(current_object, part)
        except Exception:
            return None

    return current_object


def _set_object(
    module: ModuleType, callable_path: str, original: Any, new_object: Any
) -> None:
    object_to_patch = _get_object(module, callable_path[:-1])

    original_self = getattr(original, "__self__", None)

    # Support classmethod
    if original_self and inspect.isclass(original_self):
        new_object = classmethod(new_object)

    setattr(object_to_patch, callable_path[-1], new_object)


def patch(
    module: ModuleType, module_extension: "module_extension.ModuleExtension"
) -> None:
    for callable_name, callable_extenders in module_extension.items():
        callable_path = callable_name.split(".")
        original = _get_object(module, callable_path)

        if original is None:
            continue

        new_callable = wrapper.wrap(original, callable_extenders)
        _set_object(module, callable_path, original, new_callable)



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import copy
import logging
from typing import Any, Dict

from comet_llm import logging_messages

LOGGER = logging.getLogger(__name__)


def deepmerge(
    dict1: Dict[str, Any], dict2: Dict[str, Any], max_depth: int = 10
) -> Dict[str, Any]:
    merged = copy.deepcopy(dict1)

    for key, value in dict2.items():
        if (
            key in merged
            and _is_dict(merged[key])
            and _is_dict(value)
            and max_depth > 0
        ):
            merged[key] = deepmerge(merged[key], value, max_depth=max_depth - 1)
        else:
            if key in merged:
                LOGGER.debug(
                    logging_messages.METADATA_KEY_COLLISION_DURING_DEEPMERGE,
                    key,
                    merged[key],
                    value,
                )
            merged[key] = value

    return merged


def _is_dict(item: Any) -> bool:
    return isinstance(item, dict)



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

from typing import TYPE_CHECKING

from . import hooks

if TYPE_CHECKING:  # pragma: no cover
    from comet_llm.import_hooks import registry


def patch(registry: "registry.Registry") -> None:
    registry.register_before(
        "openai", "ChatCompletion.create", hooks.before_chat_completion_create
    )
    registry.register_after(
        "openai", "ChatCompletion.create", hooks.after_chat_completion_create
    )
    registry.register_after_exception(
        "openai", "ChatCompletion.create", hooks.after_exception_chat_completion_create
    )

    registry.register_before(
        "openai.resources.chat.completions",
        "Completions.create",
        hooks.before_chat_completion_create,
    )
    registry.register_after(
        "openai.resources.chat.completions",
        "Completions.create",
        hooks.after_chat_completion_create,
    )
    registry.register_after_exception(
        "openai.resources.chat.completions",
        "Completions.create",
        hooks.after_exception_chat_completion_create,
    )



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import inspect
import logging
from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Tuple, Union

import comet_llm.logging

from . import metadata

if TYPE_CHECKING:
    from openai import Stream
    from openai.openai_object import OpenAIObject
    from openai.types.chat.chat_completion import ChatCompletion

Inputs = Dict[str, Any]
Outputs = Dict[str, Any]
Metadata = Dict[str, Any]

CreateCallResult = Union[
    "ChatCompletion", "Stream", "OpenAIObject", Iterable["OpenAIObject"]
]

LOGGER = logging.getLogger(__file__)


def create_arguments_supported(kwargs: Dict[str, Any]) -> bool:
    if "messages" not in kwargs:
        return False

    return True


def parse_create_arguments(kwargs: Dict[str, Any]) -> Tuple[Inputs, Metadata]:
    kwargs_copy = kwargs.copy()
    inputs = {}

    inputs["messages"] = kwargs_copy.pop("messages")
    if "function_call" in kwargs_copy:
        inputs["function_call"] = kwargs_copy.pop("function_call")

    metadata = {"created_from": "openai", "type": "openai_chat", **kwargs_copy}

    return inputs, metadata


def parse_create_result(result: CreateCallResult) -> Tuple[Outputs, Metadata]:
    openai_version = metadata.openai_version()

    if openai_version is not None and openai_version.startswith("0."):
        return _v0_x_x__parse_create_result(result)

    return _v1_x_x__parse_create_result(result)


def _v0_x_x__parse_create_result(
    result: Union["OpenAIObject", Iterable["OpenAIObject"]]
) -> Tuple[Outputs, Metadata]:
    if inspect.isgenerator(result):
        choices = "Generation is not logged when using stream mode"
        metadata = {}
    else:
        result_dict = result.to_dict()  # type: ignore
        choices: List[Dict[str, Any]] = result_dict.pop("choices")  # type: ignore
        metadata = result_dict

    outputs = {"choices": choices}

    if "model" in metadata:
        metadata["output_model"] = metadata.pop("model")

    return outputs, metadata


def _v1_x_x__parse_create_result(
    result: Union["ChatCompletion", "Stream"]
) -> Tuple[Outputs, Metadata]:
    stream_mode = not hasattr(result, "model_dump")
    if stream_mode:
        choices = "Generation is not logged when using stream mode"
        metadata = {}
    else:
        result_dict = result.model_dump()
        choices: List[Dict[str, Any]] = result_dict.pop("choices")  # type: ignore
        metadata = result_dict

    outputs = {"choices": choices}

    if "model" in metadata:
        metadata["output_model"] = metadata.pop("model")

    return outputs, metadata



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************
import json
from typing import Optional

import requests  # type: ignore

from .. import backend_error_codes, logging_messages

SDK_ERROR_CODES_LOGGING_MESSAGE = {
    backend_error_codes.UNABLE_TO_LOG_TO_NON_LLM_PROJECT: logging_messages.UNABLE_TO_LOG_TO_NON_LLM_PROJECT
}


def handle(response: requests.Response) -> Optional[str]:
    sdk_error_code = json.loads(response.text)["sdk_error_code"]
    return SDK_ERROR_CODES_LOGGING_MESSAGE.get(sdk_error_code)



import contextlib
import os


@contextlib.contextmanager
def environ(env):
    """Temporarily set environment variables inside the context manager and
    fully restore previous environment afterwards
    """
    original_env = {key: os.getenv(key) for key in env}
    os.environ.update(env)
    try:
        yield

    finally:
        for key, value in original_env.items():
            if value is None:
                del os.environ[key]
            else:
                os.environ[key] = value


# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

import logging
from typing import TYPE_CHECKING, Any, Callable, Dict, Iterator, Tuple

import comet_llm.logging
from comet_llm import config, experiment_info
from comet_llm.chains import api as chains_api, chain, span, state as chains_state

from . import chat_completion_parsers, context

if TYPE_CHECKING:  # pragma: no cover
    import comet_ml

LOGGER = logging.getLogger(__name__)


_chat_completion_error_logger = comet_llm.logging.log_message_on_error(
    LOGGER,
    logging_level=logging.DEBUG,
    message="Failed to log ChatCompletion.create call data",
)


@_chat_completion_error_logger
def before_chat_completion_create(original: Callable, *args, **kwargs) -> None:  # type: ignore
    if not config.is_ready():
        return

    if not chat_completion_parsers.create_arguments_supported(kwargs):
        return

    inputs, metadata = chat_completion_parsers.parse_create_arguments(kwargs)

    if chains_state.global_chain_exists():
        chain_ = chains_state.get_global_chain()
    else:
        chain_ = chain.Chain(
            inputs=inputs,
            metadata=metadata,
            experiment_info=experiment_info.get(),
            others={"Created from": "openai-llm"},
        )
        context.CONTEXT.chain = chain_

    span_ = span.Span(inputs=inputs, metadata=metadata, category="llm")

    span_.__api__start__(chain=chain_)

    context.CONTEXT.span = span_


@_chat_completion_error_logger
@context.clear_on_end
def after_chat_completion_create(original, return_value, *args, **kwargs) -> None:  # type: ignore
    if not config.is_ready():
        return

    if context.CONTEXT.span is None:
        return

    outputs, metadata = chat_completion_parsers.parse_create_result(return_value)

    span_ = context.CONTEXT.span
    span_.set_outputs(outputs=outputs, metadata=metadata)
    span_.__api__end__()

    if context.CONTEXT.chain is not None:
        chain_ = context.CONTEXT.chain
        chain_.set_outputs(outputs=outputs, metadata=metadata)
        chains_api.log_chain(chain_)


@_chat_completion_error_logger
@context.clear_on_end
def after_exception_chat_completion_create(  # type: ignore
    original: Callable, exception: Exception, *args, **kwargs
) -> None:
    # the required logic is inside clear_on_end decorator
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at http://www.comet.ml
#  Copyright (C) 2015-2021 Comet ML INC
#  This source code is licensed under the MIT license found in the
#  LICENSE file in the root directory of this package.
# *******************************************************

from pathlib import Path

from setuptools import find_packages, setup

requirements = [
    "comet_ml",
    "dataclasses; python_version<'3.7.0'",
    "flatten-dict",
    "requests",
    "types-requests",
]
project_urls = {"Source code": "https://github.com/comet-ml/comet-llm"}
this_directory = Path(__file__).parent
long_description = (this_directory / ".github" / "PACKAGE_README.md").read_text(
    encoding="utf-8"
)


setup(
    author="Comet ML Inc.",
    author_email="mail@comet.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    description="Comet logger for LLM",
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="comet_llm",
    name="comet_llm",
    packages=find_packages("src"),
    package_dir={"": "src"},
    url="https://www.comet.com",
    project_urls=project_urls,
    version="2.0.1",
    zip_safe=False,
    license="MIT",
)
